/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: Olivier Lobry (olivier.lobry@francetelecom.com)
 * Date: 15 juin 2004
 * Time: 17:54:11
 */

package org.objectweb.perseus.concurrency.distributed.globallock.api;


/**
 * This interface is used to sychronize with lock requests in asynchronous mode.
 * It is returned by the <code>upgrade</code> method of the
 * <code>GlobalLock</code> interface. This can be used to ensure FIFO handling
 * of requests when implementing multilevel lock management.
 * @see GlobalLock
 */

public interface GlobalLockWaiter {

	/**
	 * Wait until the lock is granted or refused.
	 * @param timeout max time to wait
	 * @return true if the lock is granted, false if refused
	 * @throws InterruptedException if the thread has been interrupted
	 */
	boolean waitLock(long timeout) throws InterruptedException;


	/**
	 * Signal that the lock granting has been handled by the waiter and that
	 * next waiters can have their requests granted if possible
	 */
	void signalHandled();

	/**
	 * Get the requested lock level
	 * @return the requested lock level
	 */
	byte getLockLevel();

}
