package org.objectweb.perseus.concurrency.distributed.globallock.lib;

import org.objectweb.perseus.distribution.api.DistResUserService;
import org.objectweb.perseus.distribution.api.DistResServiceManager;
import org.objectweb.perseus.distribution.api.DistResCoordinatorFactory;
import org.objectweb.perseus.concurrency.distributed.globallock.api.GlobalLock;
import org.objectweb.perseus.concurrency.distributed.globallock.api.GlobalLockManager;
import org.objectweb.perseus.concurrency.distributed.globallock.api.GlobalLockException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.NoSuchInterfaceException;

import java.io.Serializable;
import java.util.Map;
import java.util.HashMap;

/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: olivier.lobry@francetelecom.com
 * Date: 2 juin 2004
 * Time: 17:56:43
 */
public class BasicGlobalLockManager
		implements GlobalLockManager, BindingController, LifeCycleController {

	public final static String SERVICE_MGR_BINDING = "dist-res-service-manager";
	public final static String COORD_FACTORY_BINDING = "dist-res-coord-factory";

	// required interfaces
	protected DistResServiceManager drsm;
	protected DistResCoordinatorFactory drcf;

	// life cycle state
	protected String state = LifeCycleController.STOPPED;

	protected DistResUserService drus;
	protected Map globalLocks;
	protected Serializable nodeId;
/*	public BasicGlobalLockManager(DistResServiceManager drsm, LockValue lockValue) {
		this(drsm, new GlobalLockCoordinatorFactory(lockValue));
	} */
/*	public BasicGlobalLockManager(DistResServiceManager drsm,
			DistResCoordinatorFactory drcf) {
		this.nodeId = drsm.newLocalNode(drcf);
		this.drus = drsm.getUserService(nodeId);
	}
*/
	public BasicGlobalLockManager() {
	}

	// BindingController interface
	public String[] listFc() {
		return new String[] {SERVICE_MGR_BINDING, COORD_FACTORY_BINDING};
	}

	public Object lookupFc(String s) throws NoSuchInterfaceException {
		if (SERVICE_MGR_BINDING.equals(s)) {
			return drsm;
		} else if (COORD_FACTORY_BINDING.equals(s)) {
			return drcf;
		}
		return null;
//FIXME:		throw new NoSuchInterfaceException(s);
	}

	public void bindFc(String s, Object o) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {
		if (!LifeCycleController.STOPPED.equals(state)) {
			throw new IllegalLifeCycleException(s);
		}
		try {
			if (SERVICE_MGR_BINDING.equals(s)) {
				drsm = (DistResServiceManager) o;
			} else if (COORD_FACTORY_BINDING.equals(s)) {
				drcf = (DistResCoordinatorFactory) o;
			}
		} catch (ClassCastException e) {
			throw new IllegalBindingException(s + ":"  + e.getMessage());
		}
//FIXME:		throw new NoSuchInterfaceException(s);
	}

	public void unbindFc(String s) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {
		if (!LifeCycleController.STOPPED.equals(state)) {
			throw new IllegalLifeCycleException(s);
		}
		if (SERVICE_MGR_BINDING.equals(s)) {
			drus = null;
		} else if (SERVICE_MGR_BINDING.equals(s)) {
			drcf = null;
		}
//FIXME:		throw new NoSuchInterfaceException(s);
	}

	// LifeCycleController interface
	public String getFcState() {
		return state;
	}

	public void startFc() throws IllegalLifeCycleException {
		if (LifeCycleController.STARTED.equals(state)) {
			throw new IllegalLifeCycleException("Already started");
		}
		Object nodeId = drsm.newLocalNode(drcf);
		drus = drsm.getUserService(nodeId);
		globalLocks = new HashMap();
		state = LifeCycleController.STARTED;
	}

	public void stopFc() throws IllegalLifeCycleException {
		if (LifeCycleController.STOPPED.equals(state)) {
			throw new IllegalLifeCycleException("Already stopped");
		}
		// TODO: could release explictly all locks
		/* for (Iterator it = globalLocks.values().iterator(); it.hasNext();) {
			GlobalLockUser glu = (GlobalLockUser) it.next();
			glu.finilaze();
		}
		*/
		globalLocks = null;
		drsm.removeLocalNode(drus.getNodeId());
		drus = null;
		state = LifeCycleController.STOPPED;
	}

	// GlobalLockManager interface
	public GlobalLock getGlobalLock(Serializable oid,
							 boolean create) throws GlobalLockException {
		GlobalLock gl;
		synchronized (globalLocks) {
			gl = (GlobalLock) globalLocks.get(oid);
			if ((gl == null) && create) {
				gl = createLock(oid);
				globalLocks.put(oid, gl);
				joinUsers(oid);
			}
		}
		return gl;
	}

	// private/potected methods
	protected GlobalLock createLock(Serializable oid) {
		return new GlobalLockUser(oid, drus);
	}

	protected void joinUsers(Serializable oid) throws GlobalLockException {
		GlobalLockUser gl = (GlobalLockUser) globalLocks.get(oid);
		try {
			drus.joinUsers(oid, gl);
		} catch (Exception e) {
			throw new GlobalLockException(e);
		}
	}


	public Object getNodeId() {
		return nodeId;
	}
/*	public DistResUserService getUserService() {
		return drus;
	}
*/

}
