package org.objectweb.perseus.concurrency.distributed.globallock.lib;

/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR AE PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: Olivier Lobry (olivier.lobry@rd.francetelecom.com)
 *
 */


import java.io.Serializable;

/**
 * This class is used to send messages between the user and the coordinator(s) of a resource
 */

public class GlobalLockMessage implements Serializable {

	// From user to coordinator
	static final byte UPGRADE_REQUEST = 0;
	static final byte DOWNGRADE_NOTIFY = 1;
	static final byte UPGRADE_CANCEL = 2;

	// From coordinator to user
	static final byte UPGRADE_NOTIFY = 2;
	static final byte DOWNGRADE_REQUEST = 3;
	// static final byte DOWNGRADE_CANCEL = 4;

	byte type;
	byte lck;
	long timeout;
	Object requester; // may not be really required
	int serialNumber;
	//byte auxLck;

	GlobalLockMessage(byte type, byte lck, long timeout, Object requester,
					  int serialNumber) {
		this(type, lck, timeout, requester);
		this.serialNumber = serialNumber;
	}

	GlobalLockMessage(byte type, byte lck, long timeout, Object requester) {
		this.type = type;
		this.lck = lck;
		this.timeout = timeout;
		this.requester = requester;
		//this.auxLck = LockValue.UNDEFINED;
	}

	/* GlobalLockMessage(byte type, byte lck, byte auxLck, long timeout,
					  Object requester) {
		this.type = type;
		this.lck = lck;
		this.auxLck = auxLck;
		this.timeout = timeout;
		this.requester = requester;

	}
*/
	private static String typeToString(byte type) {
		switch (type) {
		case UPGRADE_NOTIFY: return "UPGRADE_NOTIFY";
		case UPGRADE_REQUEST: return "UPGRADE_REQUEST";
		case DOWNGRADE_NOTIFY: return "DOWNGRADE_NOTIFY";
		case DOWNGRADE_REQUEST: return "DOWNGRADE_REQUEST";
		}
		return "";
	}

	public String toString() {
		return "Dist RWLockValue Message (" + typeToString(type) +
				", lck=" + lck + ", req=" + requester + ", SN="
				+ serialNumber + ")";
	}

}
