/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR AE PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: Olivier Lobry (olivier.lobry@rd.francetelecom.com)
 *
 */

package org.objectweb.perseus.concurrency.distributed.globallock.lib;

import org.objectweb.perseus.concurrency.lib.LockValue;
import org.objectweb.perseus.concurrency.distributed.globallock.api.GlobalLockWaiter;


public class GlobalLockWaiterImpl implements GlobalLockWaiter
 {
	static final boolean trace = false;

	private byte lck;
	private Handled handled = new Handled();
	GlobalLockWaiterImpl waitFor;
	private class Handled {
		boolean ok = false;

		public synchronized void waitHandled(long timeout) throws InterruptedException {
			while (!ok) wait(timeout);
			//return ok;
		}

		public synchronized void signalHandled() {
			ok = true;
			notify();
		}
	}
	GlobalLockWaiterImpl(byte lck) {
		this.lck = lck;
	}

	public void waitHandled(long timeout) throws InterruptedException {
		handled.waitHandled(timeout);
	}

	public void signalHandled() {
		handled.signalHandled();
	}

	public synchronized boolean waitLock(long timeout)
			throws InterruptedException {
		if (trace) System.out.println("WAITER WAITS LOCK: " + this);
		if (lck != LockValue.NOLOCK) {
			if (trace) System.out.println("WAITER SLEEPS: " + this);
			wait(timeout);
			if (waitFor != null) {
				if (trace) System.out.println("TIMEOUT " + this);
				waitFor.waitHandled(timeout);
			}
			if (trace) System.out.println("WAITER WAKES UP: " + this);
			if (lck != LockValue.NOLOCK) {
				// this mean that wait exit on timeout
				if (trace) System.out.println("TIMEOUT " + this);
				lck = LockValue.NOLOCK;
				return false;
			}
		} else
			if (trace) System.out.println("WAITER PASS THROUGH: " + this);
		return true;
	}

	public synchronized boolean signalLock(byte signaledLck,
										   GlobalLockWaiterImpl waitFor) {
		if (trace) System.out.println("WAITER SIGNAL LOCK: " + this);

		if (signaledLck >= this.lck) {
			this.lck = LockValue.NOLOCK;
			this.waitFor = waitFor;
			if (trace) System.out.println("WAITER NOTIFIES ALL: " + this);
			notifyAll();
			return true;
		}
		else if (trace) System.out.println("WAITER SIGNALED W/ LOCK TOO WEAK ("
				+ signaledLck + "/" + lck + "): " + this);
		return false;
	}

	public byte getLockLevel() {
		return lck;
	}
}
