/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.concurrency.lib;

/**
 * A semaphore which can be disable at the definition time.
 *
 * @author S.Chassande-Barrioz
 */
public class Semaphore {

    private Object user;
	private boolean checkTread;
	private boolean reentrant = false;

    public boolean on;

    public Semaphore() {
        this(true);
    }

    public Semaphore(boolean on) {
		this(on, true);
	}
    public Semaphore(boolean on, boolean checkthread) {
		this(on, checkthread, false);
    }

    public Semaphore(boolean on, boolean checkthread, boolean reentrant) {
        this.on = on;
		this.checkTread = checkthread;
		this.reentrant = reentrant;
		user = null;
    }

    public void init(boolean isOn) {
        on = isOn;
        if (isOn) {
            user = null;
        }
    }

    public boolean isActive() {
        return on;
    }

    public boolean P() {
        if (on) {
            synchronized (this) {
				if (!reentrant || this.user != Thread.currentThread()) {
					while (user != null) {
						try {
							wait();
						} catch (InterruptedException e) {
						}
					}
					this.user = Thread.currentThread();
				}
            }
        }
        return on;
    }

	public Object getUser() {
		return user;
	}

    public void V() {
        if (on) {
            synchronized (this) {
				if (!checkTread || Thread.currentThread() == user) {
					user = null;
					notify();
				} else {
					new RuntimeException("Not owner expected:" + user
						+ ", found: " + Thread.currentThread());
				}
            }
        }
    }
}
