/**
 * Copyright (C) 2001-2002
 * - France Telecom R&D
 * - Laboratoire Logiciels, Systemes, Reseaux - UMR 5526, CNRS-INPG-UJF
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors:
 *
 */

package org.objectweb.perseus.concurrency.pessimistic;

import org.objectweb.perseus.concurrency.api.ConcurrencyException;
import org.objectweb.perseus.dependency.api.DependencyGraph;
import org.objectweb.util.monolog.api.Logger;

/**
 * A lock associated to an oid (see the "locks" map within the pessimistic
 * concurrency manager).
 * @author E. Bruneton, P. Dechamboux
 */
public abstract class Lock {

	protected int reservations = 0;
    public Object hints;
    public DependencyGraph dg = null;
    public Logger logger = null;
    public Object oid;

	public Lock() {
	}

	public Lock(Object hints, DependencyGraph dg) {
		this.hints = hints;
		this.dg = dg;
	}

	public synchronized void reserve() {
		reservations++;
	}
	
	/**
	 * Acquires this lock in read mode for the given context. This method
	 * blocks until the lock can be acquired in read mode by this context.
	 * @param ctxt a context.
	 */
	public abstract void readIntention(Object ctxt)
			throws ConcurrencyException;

	/**
	 * Acquires this lock in write mode for the given context. This method
	 * blocks until the lock can be acquired in write mode by this context.
	 * @param ctxt a context.
	 */
	public abstract void writeIntention(Object ctxt)
			throws ConcurrencyException;

	/**
	 * Removes the given context from the reader and writer lists of this
	 * lock.
	 * @param ctxt a context
	 * @return true if the reader and writer list are empty, after the
	 * context has been removed from these lists. In such a case, this
	 * object can be removed from the 'locks' map.
	 */
	public abstract boolean close(Object ctxt);


	public abstract byte getMax();
	
}
