package org.objectweb.perseus.concurrency.distributed.globallock;

import org.objectweb.perseus.concurrency.distributed.globallock.api.DeadLockException;
import org.objectweb.perseus.concurrency.distributed.globallock.api.GlobalLock;
import org.objectweb.perseus.concurrency.distributed.globallock.api.GlobalLockWaiter;
import org.objectweb.perseus.concurrency.distributed.globallock.lib.GlobalLockUser;
import org.objectweb.perseus.distribution.util.MessageEventListener;
import org.objectweb.perseus.distribution.util.MessageEventListenerRegistry;

import java.io.Serializable;

/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR AE PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: lobry
 * Date: 27 mai 2004
 * Time: 17:57:10
 */
public class GlobalLockInterceptor implements GlobalLock {


	static final public boolean trace = false;

	MessageEventListenerRegistry reg;

	GlobalLockUser glu;
	//DistResUser dru;
	//DistResUserService drus;
	String name;
	//UserInterceptor lowPart;

	public interface FailureListener {
		void handle(Serializable resId, Serializable nodeId);
	}

	public GlobalLockInterceptor(GlobalLockUser glu,
								 //DistResUser dru,
								 //DistResUserService drus,
								 //UserInterceptor lowPart,
								 MessageEventListenerRegistry reg) {
		this.glu = glu;
		//this.dru = dru;
		//this.drus = drus;
		//this.lowPart = lowPart;
		this.reg = reg;
	}


	public GlobalLockWaiter upgrade(byte lck, boolean sync, long timeout)
			throws DeadLockException, InterruptedException {
		GlobalLockWaiter w = glu.upgrade(lck, false, timeout);
		MessageEventListener l = reg.getListener();
		if (l != null) l.handledByUser();
		if (sync && (w != null)) {
			boolean ok = w.waitLock(timeout);
			if (!ok) {
					if (trace) trace("WAKE UP TIMED OUT (Waiting: " + lck + ")");
					throw new DeadLockException();
			}

			if (trace) trace("WAKE UP (Waiting: " + lck + ")");
			w.signalHandled();
			return null;
		} else {
			return w;
		}

	}

	public void downgrade(byte lck) {
		glu.downgrade(lck);
	}

	public byte getGrantable() {
		return glu.getGrantable();
	}

	public void uncache() {
		glu.uncache();
	}




	public MessageEventListenerRegistry getReg() {
		return reg;
	}

	public void setReg(MessageEventListenerRegistry reg) {
		this.reg = reg;
	}
	public synchronized String toString() {
		return glu.toString();
	}

	protected void trace(String s) {
		System.out.println(this + " " + s);
	}

	void setName(String s) {
		name = s;
	}

	String getName() {
		return name;
	}


}
