/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR AE PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: Olivier Lobry (olivier.lobry@rd.francetelecom.com)
 *
 */

package org.objectweb.perseus.concurrency.distributed.globallock;

import org.objectweb.perseus.util.action.AbstractLockingAction;
import org.objectweb.perseus.util.action.Action;
import org.objectweb.perseus.util.event.Event;

public class Upgrade implements Action {
	UpgradeReq ur;
	WaitNotGrantable w;

	public Upgrade(GlobalLockUser h,  byte expBehaviour,
					 byte execMode, byte lck, boolean waitNotGrantable) {

		if ((waitNotGrantable))
			w = new WaitNotGrantable(h, lck, execMode);
		ur = new UpgradeReq(h, expBehaviour, execMode, lck);

	}

	public void perform() throws Exception {
		if (w != null) w.perform();
		ur.perform();
	}

	public Event beginEvent() {
		if (w != null) return w.beginEvent();
		return ur.beginEvent();
	}

	public Event endEvent() {
		return ur.endEvent();
	}

	public Object depEvent() {
		if (w != null) return w.depEvent();
		return ur.depEvent();
	}
	public Event coordHandledEvent() {
		return ur.coordHandledEvent();
	}
	public Event userHandledEvent() {
		return ur.userHandledEvent();
	}

	class UpgradeReq extends AbstractLockingAction {
		GlobalLockUser h;
		public UpgradeReq (GlobalLockUser h, byte expBehaviour,
					 byte execMode, byte lck) {
			super(h.getName(), expBehaviour, execMode, lck);
			this.h = h;
		}

		protected void performBlockingAction() throws Exception {
			GlobalLockWaiter w = h.localUpgrade(lck, false);
			handledByUser();
			if (w != null) {
				boolean ok = w.waitLock(0);
				if (!ok) {
					if (h.trace) h.trace("WAKE UP TIMED OUT (Waiting: " + lck + ")");
					te.addException(new DeadLockException());
				}
				if (h.trace) h.trace("WAKE UP (Waiting: " + lck + ")");
				return;
			}
		}

		final public String getBrief() {
			return "U";
		}

		protected String getLockingDescription() {
			return "upgrade";
		}

	}
	public String toString() {
		return ur.toString();
	}
}
